/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - Hello World for RDMSQL */
/* Each time you run this example, a new record is created in the
 * database, then all existing records are read from the database
 * and printed.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDb/rsqlCloseDb
 *      rsqlExecDirect
 *      rsqlTransCommit
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Each of these functions returns a RSQL_ERRCODE status code, where the
 * value errSUCCESS indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDMSQL call.
 *
 */

static RSQL_ERRCODE create_database()
{
    RSQL_ERRCODE rsqlCode;
    HCONN        hDBC;
    HSTMT        hSTMT;

    /* Allocate handles */
    rsqlCode = rsqlAllocConn(&hDBC);
    rsqlCode = rsqlAllocStmt(hDBC, &hSTMT);

    /* Attempt to create the database.  If the database already exists
     * this will fail and we simply open the existing database. */
    rsqlCode = rsqlExecDirect(hSTMT, "CREATE DATABASE HelloWorld_db");
    if (rsqlCode == errSUCCESS)
    {
        rsqlExecDirect(hSTMT, "CREATE TABLE hw_table (message char(30))");
        rsqlTransCommit(hDBC);
        rsqlCloseDB(hDBC, "HelloWorld_db");
    }
    else if (rsqlCode == errDBEXISTS)
    {
        rsqlCode = errSUCCESS;
    }
    else
    {
        fprintf(stderr, "Error creating the HelloWorld database\n");
    }

    /* Free the handles */
    rsqlFreeStmt(hSTMT);
    rsqlFreeConn(hDBC);

    return rsqlCode;
}

/* =======================================================================
*/
int32_t main(void)
{
    HCONN             hDBC;
    HSTMT             hSTMT;
    RSQL_ERRCODE      rsqlCode;
    const RSQL_VALUE *result;
    uint16_t          numCols;

    create_database();

    /* Allocate handles */
    rsqlCode = rsqlAllocConn(&hDBC);
    if (rsqlCode == errSUCCESS)
    {
        rsqlCode = rsqlAllocStmt(hDBC, &hSTMT);
        if (rsqlCode == errSUCCESS)
        {
            /* Open the database in exclusive mode */
            rsqlCode = rsqlOpenDB(hDBC, "HelloWorld_db", "x");

            /* Add a new row to hw_table */
            if (rsqlCode == errSUCCESS)
            {
                rsqlCode = rsqlExecDirect(hSTMT,
                        "INSERT INTO hw_table VALUES \"Hello World\"");
                (void) rsqlTransCommit(hDBC);
            }

            if (rsqlCode == errSUCCESS)
            {
                /* Retrieve and display all rows from hw_table */
                rsqlCode = rsqlExecDirect(hSTMT, "SELECT message FROM hw_table");
                while ((rsqlCode = rsqlFetch(hSTMT, &result, &numCols)) == errSUCCESS)
                    printf("%s\n", result[0].vt.cv);
            }

            /* close the database */
            (void) rsqlCloseDB(hDBC, "HelloWorld_db");

            /* Free the statement handle */
            (void) rsqlFreeStmt(hSTMT);
        }

        /* Free the connection handle */
        (void) rsqlFreeConn(hDBC);
    }

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}
